import { createDecorator } from 'vue-class-component';
export { Options, mixins } from 'vue-class-component';

function State(value) {
    var arr = value ? value.split("/") : undefined;
    return function (target, propertyKey) {
        createDecorator(function (options, key) {
            options.computed[key] = function () {
                var data = this.$store.state;
                if (!arr) {
                    return data[key];
                }
                else {
                    arr.forEach(function (item) {
                        data = data[item];
                    });
                    return data;
                }
            };
        })(target, propertyKey);
    };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

function Action(value) {
    return function (target, propertyKey) {
        createDecorator(function (options, key) {
            options.methods[key] = function () {
                var _a;
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var actionName = value || key;
                return (_a = this.$store).dispatch.apply(_a, __spreadArray([actionName], args));
            };
        })(target, propertyKey);
    };
}

function Getter(value) {
    return function (target, propertyKey) {
        createDecorator(function (options, key) {
            options.computed[key] = function () {
                var name = value || key;
                return this.$store.getters[name];
            };
        })(target, propertyKey);
    };
}

function Mutation(value) {
    return function (target, propertyKey) {
        createDecorator(function (options, key) {
            options.methods[key] = function () {
                var _a;
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var name = value || key;
                return (_a = this.$store).commit.apply(_a, __spreadArray([name], args));
            };
        })(target, propertyKey);
    };
}

export { Action, Getter, Mutation, State };
